#include <iostream>
#include <unistd.h>
using namespace std;
#include <string>
#include <nameParser.h>

nameParser::nameParser(char *fullname)
{
  parseQuality(0);
  parseFullName(fullname);
}
void nameParser::initParser(){
  NameSpace[0]=0;
  NodeSpec[0]=0;
  NodeName[0]=0;
  NodeID[0]=0;
  ApplSpec[0]=0;
  ApplName[0]=0;
  ApplNameID[0]=0;
  ApplNameSpace[0]=0;
  ApplID[0]=0;
  Name[0]=0;
  FullName[0]=0;
}
void nameParser::parseQuality(unsigned int qual){
  quality   = qual;
  status    =  quality       & 0xff;
  type      = (quality>>8)  & 0xff;
  visibility= (quality>>16) & 0xff;
  mode      = (quality>>24) & 0xff;
}
int nameParser::buildQuality(unsigned int s, unsigned int t, unsigned int v, unsigned int m){
  status=s;
  type=t;
  visibility=v;
  mode=m;
  quality= s | (t << 8) | (v << 16) | (m << 24);
  return quality;
}
bool nameParser::checkFields(){
  if(strlen(NameSpace)==0) return false;
  if(strlen(NodeName)==0) return false;
  if(strlen(NodeID)==0) return false;
  if(strlen(ApplName)==0) return false;
  if(strlen(ApplID)==0) return false;
  if(strlen(Name)==0) return false;
  return true;
}
char* nameParser::buildFullName()
{ // check if all fields are there
  if(!checkFields()) return NULL;
  buildNodeSpec();
  buildApplSpec();
  sprintf(FullName,"%s/%s/%s/%s",NameSpace,NodeSpec,ApplSpec,Name);
  return FullName;
}
void nameParser::buildNodeSpec(){
  sprintf(NodeSpec,"%s:%s",NodeName,NodeID);
}
void nameParser::buildApplSpec(){
  sprintf(ApplNameID,"%s:%s",ApplName,ApplID);
  if(strlen(ApplNameSpace) > 0) sprintf(ApplSpec,"%s::%s",ApplNameSpace,ApplNameID);
  else strcpy(ApplSpec,ApplNameID);
}
void nameParser::setNodeSpec(char *nodespec){
  strcpy(NodeSpec,nodespec);
  parseNodeSpec();
}
void nameParser::setNodeName(char *nodename){
  strcpy(NodeName,nodename);
  buildNodeSpec();
}
void nameParser::setNodeID(char *nodeid){
  strcpy(NodeID,nodeid);
  buildNodeSpec();
}
void nameParser::setApplSpec(char *applspec){
  strcpy(ApplSpec,applspec);
  parseApplSpec();
}
void nameParser::setApplName(char *applname){
  strcpy(ApplName,applname);
  buildApplSpec();
}
void nameParser::setApplNameID(char *applnameid){
  strcpy(ApplNameID,applnameid);
  parseApplNameID();
  buildApplSpec();
}
void nameParser::setApplID(char *applid){
  strcpy(ApplID,applid);
  buildApplSpec();
}
void nameParser::setApplNameSpace(char *applnamespace){
  strcpy(ApplNameSpace,applnamespace);
  buildApplSpec();
}
bool nameParser::parseApplSpec(){
  char *p1;
  p1=strstr(ApplSpec,"::");
  if(p1){  // namespace specified
    strcpy(ApplNameSpace,ApplSpec);
    *p1++ = 0; p1++;
    strcpy(ApplNameID,p1);
  }
  else { // no namespace
    strcpy(ApplNameID,ApplSpec);
    ApplNameSpace[0]=0;
  }
  return parseApplNameID();
}
bool nameParser::parseApplNameID(){
  char *p2;
  strcpy(ApplName,ApplNameID);
  p2=ApplName;
  while(1)  {
    while((*p2 != 0)&(*p2 != ':'))p2++;
    if(*p2 == 0) break; // error
    *p2++ = 0;
    strcpy(ApplID,p2);
    return true;
  }
  return false;
}
bool nameParser::parseNodeSpec(){
  char *p2;
  strcpy(NodeName,NodeSpec);
  p2=NodeName;
  while(1)  {
    while((*p2 != 0)&(*p2 != ':'))p2++;
    if(*p2 == 0) break; // error
    *p2++ = 0;
    strcpy(NodeID,p2);
    return true;
  }
  return false;
}

bool nameParser::parseFullName(char *fullname){
  char *p1,*p2;
  char fn[512];
  if(fullname) strcpy(fn,fullname);
  else strcpy(fn,FullName);
  initParser();
  p1=fn;
  p2=fn;
  // get fields between the slashes
  while(1){
    while((*p2 != 0)&(*p2 != '/'))p2++;
    if(*p2 == 0) break; // error
    *p2++ =0;
    strcpy(NameSpace,p1);
    p1=p2;
    while((*p2 != 0)&(*p2 != '/'))p2++;
    if(*p2 == 0) break; // error
    *p2++ =0;
    strcpy(NodeSpec,p1);
    p1=p2;
    while((*p2 != 0)&(*p2 != '/'))p2++;
    if(*p2 == 0) break; // error
    *p2++ =0;
    strcpy(ApplSpec,p1);
    p1=p2;
    strcpy(Name,p1);
    // parse subfields
    if(!parseNodeSpec())break;
    if(!parseApplSpec())break;
    buildFullName();
    return true;
  }
  cout << "Error: " << 
  NameSpace << "/" <<
  NodeName << ":" <<
  NodeID << "/" <<
  ApplNameSpace << "::" <<
  ApplName << ":" <<
  ApplID << "/" <<
  Name << endl;
  strcpy(FullName,"");
  return false;
}

void nameParser::printFields() {
  cout << "Full:  " << FullName << endl;
  cout << "Fields: " << 
  NameSpace << "/" <<
  NodeName << ":" <<
  NodeID << "/" <<
  ApplNameSpace << "::" <<
  ApplName << ":" <<
  ApplID << "/" <<
  Name << endl;

  cout << "NodeSpec: " << 
  NodeSpec << "  ApplSpec: " <<
  ApplSpec << "  ApplNameID: " <<
  ApplNameID << endl;
}
void nameParser::newCommand(char* name, char* scope){
  char temp[128];
  finalized=false;
  strcpy(XmlString,"<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
  sprintf(temp,"<command name=\"%s\" scope=\"%s\">\n",name,scope);
  strcat(XmlString,temp);
}
void nameParser::addArgument(char* name, char* type, char* value, char* required){
  char temp[128];
  if(!finalized) {
    sprintf(temp,"<argument name=\"%s\" type=\"%s\" value=\"%s\" required=\"%s\"/>\n",name,type,value,required);
    strcat(XmlString,temp);
  }
}
char * nameParser::getXmlString(){
  if(!finalized) strcat(XmlString,"</command>\n");
  finalized=true;
  return XmlString;

}

